﻿

Imports System.Net
Imports System.Net.WebRequestMethods
Imports System.IO

Module Module_FtpUpload

    Friend Working As Boolean = False
    Friend AbortFlag As Boolean
    Friend Delegate Sub ProgressCallBack(ByVal params As ProgressParams)

    Friend Structure ProgressParams
        Dim progress As Double
        Dim fileLength As Long
        Dim bytesRead As Int32
        Dim bytesTransferred As Long
        Dim secondsTotal As Double
        Dim secondsElapsed As Double
        Dim secondsRemaining As Double
        Dim kbps As Double
        Dim timeStart As Date
        Dim elapsedTime As Double
        Dim totalTime As Double
    End Structure


    ' ===================================================================================================
    '   FTP_Upload
    ' ===================================================================================================
    Friend Sub FTP_Upload(ByVal LocalFile As String, _
                          ByVal ServerFile As String, _
                          ByVal UserName As String, _
                          ByVal Password As String, _
                          Optional ByVal Callback As ProgressCallBack = Nothing)

        'If Working Then Exit Sub
        'Working = True
        'Application.UseWaitCursor = True
        'Cursor.Current = Cursors.WaitCursor

        Dim params As ProgressParams
        Dim FileStream As FileStream = Nothing
        Dim FtpStream As Stream = Nothing
        Dim FtpRequest As FtpWebRequest = Nothing

        With params

            Try
                ' --------------------------------------------------------------------- init buffer
                Dim bytesPerBlock As Integer = 4096
                Dim buffer(bytesPerBlock - 1) As Byte

                ' --------------------------------------------------------------------- FPT request (upload)
                FtpRequest = TryCast(WebRequest.Create(ServerFile), FtpWebRequest)
                FtpRequest.Method = WebRequestMethods.Ftp.UploadFile
                FtpRequest.Credentials = New NetworkCredential(UserName, Password)
                FtpRequest.UsePassive = True
                FtpRequest.UseBinary = True
                FtpRequest.KeepAlive = False ' <<< close the connection when done

                ' --------------------------------------------------------------------- open the FTP stream
                FtpStream = FtpRequest.GetRequestStream()

                ' --------------------------------------------------------------------- open the file stream
                FileStream = New FileStream(LocalFile, _
                                            FileMode.Open, _
                                            FileAccess.Read)

                ' --------------------------------------------------------------------- start the write loop
                AbortFlag = False
                .fileLength = FileStream.Length
                .bytesTransferred = 0
                .timeStart = Date.Now
                Dim oldseconds As Double = 0
                Do
                    ' ----------------------------------------------------------------- read from the file
                    .bytesRead = FileStream.Read(buffer, 0, buffer.Length)
                    '
                    If .bytesRead > 0 Then

                        ' ------------------------------------------------------------- update total bytes
                        .bytesTransferred += .bytesRead

                        ' ------------------------------------------------------------- write to stream
                        FtpStream.Write(buffer, 0, .bytesRead)

                        ' ------------------------------------------------------------- update progress
                        If Callback IsNot Nothing Then
                            .progress = (.bytesTransferred * 100) \ .fileLength
                            .secondsElapsed = (Date.Now - .timeStart).TotalSeconds
                            .secondsTotal += 0.3 * (.secondsElapsed * .fileLength / .bytesTransferred - .secondsTotal)
                            .secondsRemaining = .secondsTotal - .secondsElapsed
                            If .secondsTotal > 0 Then .kbps = 8 / 1000 * .fileLength / .secondsTotal
                            If .secondsElapsed >= oldseconds + 0.1 Then
                                oldseconds = .secondsElapsed
                                Callback(params)
                            End If
                        End If
                    End If

                    If AbortFlag Then FtpRequest.Abort()

                    Application.DoEvents()

                Loop While .bytesRead > 0

            Catch ex As Exception
                'MsgBox(ex.Message)
                If FtpRequest IsNot Nothing Then FtpRequest.Abort()
                Callback = Nothing

            Finally
                If Callback IsNot Nothing Then
                    .progress = 0
                    Callback(params)
                End If
                If FileStream IsNot Nothing Then FileStream.Close()
                If FtpStream IsNot Nothing Then FtpStream.Close()

            End Try

        End With

        'Application.UseWaitCursor = False
        'Cursor.Current = Cursors.Default
        'Working = False
    End Sub




End Module
